'***************
'*Retrosys Pong*
'* Version 1.0 *
'***************
'(C) 2007 - Shayne Helms

CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  _stack = ($3000 + $3000 + 100) >> 2   'accomodate display memory and stack

  x_tiles = 16
  y_tiles = 12

  paramcount = 14       
  bitmap_base = $2000
  display_base = $5000

  lines = 5
  thickness = 2
  

VAR

  long  tv_status     '0/1/2 = off/visible/invisible           read-only
  long  tv_enable     '0/? = off/on                            write-only
  long  tv_pins       '%ppmmm = pins                           write-only
  long  tv_mode       '%ccinp = chroma,interlace,ntsc/pal,swap write-only
  long  tv_screen     'pointer to screen (words)               write-only
  long  tv_colors     'pointer to colors (longs)               write-only               
  long  tv_hc         'horizontal cells                        write-only
  long  tv_vc         'vertical cells                          write-only
  long  tv_hx         'horizontal cell expansion               write-only
  long  tv_vx         'vertical cell expansion                 write-only
  long  tv_ho         'horizontal offset                       write-only
  long  tv_vo         'vertical offset                         write-only
  long  tv_broadcast  'broadcast frequency (Hz)                write-only
  long  tv_auralcog   'aural fm cog                            write-only

  word  screen[x_tiles * y_tiles]
  long  colors[64]

  byte  x[lines]
  byte  y[lines]
  byte  xs[lines]
  byte  ys[lines]
  word  p1_score
  word  p2_score
  byte  p1_y
  byte  p2_y
  byte  ball_x
  byte  ball_y
  byte  x_step
  byte  y_step
  byte  score_inc

OBJ
 aud : "BasicSound_1_0"
 pad : "Controller_1_0"
 gr  : "Graphics"
 tv  : "tv"

PUB Startup | i,dx,dy
  pad.start

  'start tv
  longmove(@tv_status, @tvparams, paramcount)
  tv_screen := @screen
  tv_colors := @colors
  tv.start(@tv_status)

  'init colors
  repeat i from 0 to 63
    colors[i] := $00001010 * (i+4) & $F + $2B060C02

  'init tile screen
  repeat dx from 0 to tv_hc - 1
    repeat dy from 0 to tv_vc - 1
      screen[dy * tv_hc + dx] := display_base >> 6 + dy + dx * tv_vc + ((dy & $3F) << 10)

  'start and setup graphics
  gr.start
  gr.setup(16, 12, 0, 0, bitmap_base)

  repeat
   repeat until pad.p1_start or pad.p2_start 'Wait on title screen for player to start
    gr.clear
    gr.textmode(1,1,6,5)
    gr.colorwidth(3,0)
    gr.box(70,50,120,60)
    gr.colorwidth(2,0)
    gr.text(128,96,@intro)
    gr.text(128,86,@intro2)
    gr.text(128,76,@intro3)
    gr.text(128,66,@intro4)
    gr.copy(display_base)
   p1_score := p2_score := "0"
   if pad.p1_start
    x_step := -2
    y_step := -2
   if pad.p2_start
    x_step := 2
    y_step := -2   
   repeat until p1_score == "9" or p2_score == "9" 'Main Game loop until someone gets a score of 9
    score_inc := 0 'Reset score distributor
    ball_x := 128 'Re-center the ball
    ball_y := 96
    p1_y := p2_y := 96 'Re-center paddles
    DrawScreen
    if p1_score < p2_score 'Determine who serves based on the score
     x_step := -2
     y_step := -2
    if p2_score < p1_score
     x_step := 2
     y_step := -2
    Delay(3000) 'Wait for 3 seconds before the ball is served
    repeat until score_inc <> 0
     MoveBall 
     MovePaddles
     DrawScreen
     Delay(10)
    if !aud.active
     aud.start(400, 100)
    if score_inc == 1
     p1_score ++
    if score_inc == 2
     p2_score ++
   DrawScreen
   Delay(4000)
PUB MovePaddles
 if pad.p1_active 'Only take input if the controller is active (plugged in)
  if pad.P1_UP  
   p1_y += 4
  if pad.P1_DOWN 
   p1_y -= 4
  if pad.p1_start 'Pause
   Delay(250)
   repeat until pad.p1_start
   Delay(250) 
 if pad.p2_active
  if pad.P2_UP 
   p2_y += 4
  if pad.p2_down 
   p2_y -= 4
  if pad.p2_start
   Delay(250)
   repeat until pad.p2_start
   Delay(250) 
 if p1_y => 180 'Motion Limits
  p1_y := 180
 if p1_y  =< 12
  p1_y :=12
 if p2_y => 180
  p2_y := 180
 if p2_y =< 12
  p2_y := 12

PUB MoveBall
 ''Checks for Ball Collisions then moves the ball.
 if ball_x =< 6 and (p1_y-12) < ball_y and  (p1_y+12) > ball_y 'If ball hits p1_paddle
  x_step := -x_step
  if !aud.active
   aud.start(3000,100) 
 if ball_x => 250 and (p2_y-12) < ball_y and (p2_y+12) > ball_y 'If ball hits p2_paddle 
  x_step := -x_step 
  if !aud.active
   aud.start(3000,100) 
 if ball_y => 190 or ball_y =< 2 'Ball Vertical limits
  y_step := -y_step
  if !aud.active
   aud.start(1000,100) 
 if ball_x =< 4 'Ball got past player 1 
  score_inc := 2
 if ball_x => 252 'Ball got past player 2 
  score_inc := 1
 ball_x += x_step 'Move the ball
 ball_y += y_step
return
PUB Delay(DelayMS)
waitcnt(clkfreq / 1000 * DelayMS + cnt)           '  Wait for Delay Cycles

PUB DrawScreen
    gr.clear
    gr.textmode(5,5,6,5)
    gr.colorwidth(1,4)
    gr.text(20,165,@p1_score)
    gr.text(236,165,@p2_score)
    gr.colorwidth(1,0)
    gr.plot(128,96)
    gr.line(128,0)
    gr.line(128,192)
    gr.pix(2,p1_y,1,@paddle)
    gr.pix(254,p2_y,1,@paddle)
    gr.pix(ball_x,ball_y,0,@ball)       
    gr.copy(display_base)

DAT

tvparams                long    0               'status
                        long    1               'enable
                        long    %011_0000       'pins
                        long    %0000           'mode
                        long    0               'screen
                        long    0               'colors
                        long    x_tiles         'hc
                        long    y_tiles         'vc
                        long    10              'hx
                        long    1               'vx
                        long    0               'ho
                        long    1               'vo
                        long    0'55_250_000      'broadcast CH2
                        long    0               'auralcog

paddle                  word
                        byte 2,3,7,1
                        word %%01111111,%%11111100
                        word %%12222222,%%22222210
                        word %%01111111,%%11111100

ball                    word
                        byte 1,7,3,3
                        word %%00000000
                        word %%00333000
                        word %%03111300
                        word %%03111300
                        word %%03111300
                        word %%00333000
                        word %%00000000

intro                   byte " Retrosys PONG",0

intro2                  byte "  Press Start",0

intro3                  byte " For a New Game",0

intro4                  byte "First to 9 wins",0                 